class UpdateScoreFromOfferJob < ActiveJob::Base
   queue_as :x_hermes_score_offers
  unique :until_executed

  def perform(offer_id)
    i=0
    eligible_applicant_ids = []
    offer = Hermes::HousingOffer.only(:id, :effort_rate, :lessor_id, :typology, :eligibility_query).find_by(id: offer_id)
    unless offer.formated_eligibility.blank?  
      eligible_applicant_ids = Hermes::HousingRequest.where(is_active: true, in_progress: true, **offer.formated_eligibility).pluck(:main_applicant_id).map(&:to_s)
    end
    total = eligible_applicant_ids.length
    return unless eligible_applicant_ids.present?

    UpdateOfferEligibleNbJob.perform_later(offer_id.to_s)

    found_or_created_scores = []
    eligible_applicant_ids.each do |applicant_id|
      applicant = Hermes::Applicant.only(:id).find_by(id: applicant_id)

      next unless offer && applicant

      score = Hermes::Score.find_or_create_by(housing_offer_id: offer_id, applicant_id: applicant_id, is_active: true) do
        # Enter here only if score was created
        # Temporary_disabled_devops_7390
        # channels = HERMES_CONFIG["housing_offer_notifications"] ? ['email'] : []
        # offer.notify(key: NOTIFICATION_KEYS[:housing_offer][:new_eligible_offer], channels: channels)
      end
      score.process

      Rails.logger.debug "calc score for offer [#{i}/#{total}] #{offer_id} applicant : #{applicant_id} score : #{score.id}"
      found_or_created_scores << score.id.to_s
    end

    other_scores = Hermes::Score.where(housing_offer_id: offer_id, id: { :$nin => found_or_created_scores }, is_active: true)
    other_scores.set(is_active: false, updated_at: DateTime.current)
    UpdateRankJob.perform_later(offer_id)

    # Remove application data
    housing_application_ids = other_scores.where(:housing_application_id.ne => nil).pluck(:housing_application_id).uniq
    Hermes::HousingApplication.removeScoreData(housing_application_ids)

    Rails.logger.debug "#{other_scores.count} scores have been set to inactive for housing_offer : #{offer_id}"
  end
end
