# frozen_string_literal: true

# exemple d'utilisation
=begin
ha = Hermes::HousingApplication.first
so=Hermes::BoClient::ApplicationTask.new(
    {
                                                 housing_request_id: ha.housing_request_id,
                                                 housing_offer_id: ha.housing_offer_id,
                                             }
)
so.to_xml
ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
=end

module Hermes
  module BoClient
    class UpdatePJTask < BaseTask
      
      def initialize(options = {})
        @xsd = 'integrationPJ.xsd'
        super(options)
      end
      
      endpoint {'ws/rest/als/demande/v1/pieceJustificative/modifier'}
      
      task_name 'application'
      type_flux 'BIEN'
      mapping 'Hermes::BoClient::Mappings::HousingRequestMap'
      method_name "put"
      params *%i[
      numUnique
      ]
      def to_xml
        namespaces = {
            'xmlns:ns2' => 'http://v1.als.ikos.webservice.immobilier.sopra/pieceJustificative'
        }
        builder = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |b|
          b['ns2'].integrationPieceJustificative(namespaces) do
            b.metadonnees do
              b.parent.namespace = nil
              b.numeroUnique mapping.numUnique
              b.codeGuichet mapping.numGuichet
              b.idPiece mapping.idPiece
              b.codeCategorie mapping.codeCategoriePiece
              b.type mapping.typePiece
              b.indicePertinence mapping.indicePertinence
              b.dateFonctionnelle mapping.dateFonctionnellePiece
              b.sensibilite mapping.sensibilitePiece
              b.documentInterne mapping.documentInternePiece
              b.dateCreation mapping.dateCreationPiece
              b.dateDerniereAction mapping.dateDerniereActionPiece
              b.creationAuteur
              b.nomFichier mapping.nomFichierPiece
            end
            b.contenuPJ do
              b.parent.namespace = nil
              b.nomFichier mapping.nomFichierPiece
              b.contenu mapping.contenuFichier
            end
            
          end
        end
        builder.to_xml
      end
    end
  end
end
